@@ -1,6 +1,8 @@
+double unitBF21fun(int n, int k2, int k0, double Q);
 double gBF21fun(int n, int k2, int k0, double Q);
 double flsBF21fun(int p, int n, int k2, int k0, double Q);
 double RobustBF21fun(int n, int k2, int k0, double Q);
 double LiangBF21fun(int n, int k2, int k0, double Q);
 double ZSBF21fun(int n, int k2, int k0, double Q);
+double Robust2BF21fun(int n, int k2, int k0, double Q);
 
